#include "mainwindow.h"
#include "qglobalshortcut.h"
#include <QWidget>
#include <QLabel>
#include <QLineEdit>
#include <QGridLayout>
#include <QAction>
#include <QMenu>
#include <QMenuBar>
#include <QKeySequence>
#include <QKeySequenceEdit>
#include <QApplication>
#include <QPushButton>
#include <QMessageBox>

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent), mEdit{}, mHide{}, mFull{}
{
    auto menu = this->menuBar();
    auto preference = menu->addAction("Preference");
    connect(preference, &QAction::triggered, [this] {
        this->showEditor();
    });
    preference->setShortcut(QKeySequence::fromString("Ctrl+P"));
    auto quit = menu->addAction("Quit");
    connect(quit, &QAction::triggered, [this] {
        qApp->quit();
    });
    quit->setShortcut(QKeySequence::fromString("Ctrl+Q"));
    initEditor();
}

MainWindow::~MainWindow()
{
    delete mEdit;
}

void MainWindow::showEditor()
{
    mEdit->show();
}

void MainWindow::initEditor()
{
    mEdit = new QWidget{};
    QLabel* hide = new QLabel{};
    hide->setText("hide/show");
    mHide = new KeyEdit{mEdit};

    QLabel* full = new QLabel{};
    full->setText("fullscreen/restore");
    mFull = new KeyEdit{mEdit};

    QPushButton* acc = new QPushButton{};
    acc->setText("Ok");

    QPushButton* rej = new QPushButton{};
    rej->setText("Cancel");

    QGridLayout* g = new QGridLayout{};
    g->addWidget(hide, 0, 0);
    g->addWidget(mHide, 0, 1);
    g->addWidget(full, 1, 0);
    g->addWidget(mFull, 1, 1);


    g->addWidget(acc, 2, 0);
    g->addWidget(rej, 2, 1);

    connect(acc, &QPushButton::clicked, mEdit, [this] {
        mEdit->hide();
        mFull->setKey();
        mHide->setKey();
    });
    connect(rej, &QPushButton::clicked, mEdit, [this] {
        mHide->clear();
        mFull->clear();
        mEdit->hide();
    });

    connect(mHide, &KeyEdit::actived, [this] {
        if(this->isHidden())
        {
            this->activateWindow();
            this->raise();
            this->show();
        }
        else
        {
            this->hide();
        }
    });
    connect(mFull, &KeyEdit::actived, [this] {
        if(this->isFullScreen())
        {
            this->showNormal();
        }
        else
        {
            this->showFullScreen();
        }
    });

    mEdit->setLayout(g);
    mEdit->resize(mEdit->sizeHint());
    mEdit->hide();
}

KeyEdit::KeyEdit(QWidget *parent)
    : QKeySequenceEdit{parent}
{
    mGhk = new QGlobalShortcut{this};
    connect(mGhk, &QGlobalShortcut::activated, [this] {
        this->actived(mGhk->key());
    });
}

KeyEdit::~KeyEdit()
{
    delete mGhk;
}

void KeyEdit::clear()
{
    mGhk->unsetKey();
    QKeySequenceEdit::clear();
}

void KeyEdit::setKey()
{
    mGhk->unsetKey();
    mGhk->setKey(this->keySequence());
}
